# Pyntor::Nowait - Do not wait for a key stroke, but proceed immideately
# Copyright (C) 2006 Josef Spillner <josef@coolprojects.org>
# Published under GNU GPL conditions

import pygame
from pygame.locals import *

class Nowait:
	def init(self, options):
		self.pages = 0

	def interactive(self, event):
		return "next"

component = Nowait()

parameters = ()

metainfo = {
	"version": "0.2",
	"author": "Josef Spillner",
	"authoremail": "<josef@coolprojects.org>",
	"licence": "GPL"
}

doc = """
Nowait: does not wait, proceed immediately, thus forcing a page break

Usage: simply 'nowait'

Display: none

Interactivity: none (proceeds to next slide immediately)
"""

